#!/usr/bin/env python

##Copyright 2008-2015 Jelle Feringa (jelleferinga@gmail.com)
##
##This file is part of pythonOCC.
##
##pythonOCC is free software: you can redistribute it and/or modify
##it under the terms of the GNU Lesser General Public License as published by
##the Free Software Foundation, either version 3 of the License, or
##(at your option) any later version.
##
##pythonOCC is distributed in the hope that it will be useful,
##but WITHOUT ANY WARRANTY; without even the implied warranty of
##MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##GNU Lesser General Public License for more details.
##
##You should have received a copy of the GNU Lesser General Public License
##along with pythonOCC.  If not, see <http://www.gnu.org/licenses/>.

from typing import Any, Iterable, Iterator, List, Optional, Tuple

from OCC.Core.BRep import BRep_Tool, BRep_Builder
from OCC.Core.BRepTools import BRepTools_WireExplorer
from OCC.Core.gp import gp_Ax2, gp_Dir, gp_Pnt
from OCC.Core.HLRBRep import HLRBRep_Algo, HLRBRep_HLRToShape
from OCC.Core.HLRAlgo import HLRAlgo_Projector
from OCC.Core.TopAbs import (TopAbs_VERTEX, TopAbs_EDGE, TopAbs_FACE, TopAbs_WIRE,
                             TopAbs_SHELL, TopAbs_SOLID, TopAbs_COMPOUND,
                             TopAbs_COMPSOLID, TopAbs_ShapeEnum)
from OCC.Core.TopExp import TopExp_Explorer, topexp_MapShapesAndAncestors
from OCC.Core.TopTools import (TopTools_ListIteratorOfListOfShape,
                               TopTools_IndexedDataMapOfShapeListOfShape)
from OCC.Core.TopoDS import (topods, TopoDS_Wire, TopoDS_Vertex, TopoDS_Edge,
                             TopoDS_Face, TopoDS_Shell, TopoDS_Solid, TopoDS_Shape,
                             TopoDS_Compound, TopoDS_CompSolid, topods_Edge,
                             topods_Vertex, TopoDS_Iterator)
from OCC.Core.GCPnts import (GCPnts_UniformAbscissa,
                             GCPnts_QuasiUniformDeflection,
                             GCPnts_UniformDeflection)
from OCC.Core.BRepAdaptor import BRepAdaptor_Curve


class WireExplorer:
    '''
    Wire traversal
    '''
    def __init__(self, wire: TopoDS_Wire) -> None:
        if not isinstance(wire, TopoDS_Wire):
            raise AssertionError('not a TopoDS_Wire')
        self.wire = wire
        self.wire_explorer = BRepTools_WireExplorer(self.wire)
        self.done = False

    def _reinitialize(self) -> None:
        self.wire_explorer = BRepTools_WireExplorer(self.wire)
        self.done = False

    def _loop_topo(self, edges: Optional[bool] = True) -> Iterator[Any]:
        if self.done:
            self._reinitialize()
        topologyType = topods_Edge if edges else topods_Vertex
        seq = []

        while self.wire_explorer.More():
            # loop edges
            if edges:
                current_item = self.wire_explorer.Current()
            # loop vertices
            else:
                current_item = self.wire_explorer.CurrentVertex()
            seq.append(topologyType(current_item))
            self.wire_explorer.Next()

        self.done = True
        return iter(seq)

    def ordered_edges(self) -> Iterator[TopoDS_Edge]:
        return self._loop_topo(edges=True)

    def ordered_vertices(self) -> Iterator[TopoDS_Vertex]:
        return self._loop_topo(edges=False)


class TopologyExplorer:
    '''
    Topology traversal
    '''

    def __init__(self, myShape: TopoDS_Shape, ignore_orientation: Optional[bool]=True) -> None:
        """
        implements topology traversal from any TopoDS_Shape
        this class lets you find how various topological entities are connected from one to another
        find the faces connected to an edge, find the vertices this edge is made from, get all faces connected to
        a vertex, and find out how many topological elements are connected from a source

        *note* when traversing TopoDS_Wire entities, its advised to use the specialized
        ``WireExplorer`` class, which will return the vertices / edges in the expected order

        :param myShape: the shape which topology will be traversed

        :param ignore_orientation: filter out TopoDS_* entities of similar TShape but different Orientation

        for instance, a cube has 24 edges, 4 edges for each of 6 faces

        that results in 48 vertices, while there are only 8 vertices that have a unique
        geometric coordinate

        in certain cases ( computing a graph from the topology ) its preferable to return
        topological entities that share similar geometry, though differ in orientation
        by setting the ``ignore_orientation`` variable
        to True, in case of a cube, just 12 edges and only 8 vertices will be returned

        for further reference see TopoDS_Shape IsEqual / IsSame methods

        """
        self.myShape = myShape
        self.ignore_orientation = ignore_orientation

        # the topoFactory dicts maps topology types and functions that can
        # create this topology
        self.topoFactory = {
            TopAbs_VERTEX: topods.Vertex,
            TopAbs_EDGE: topods.Edge,
            TopAbs_FACE: topods.Face,
            TopAbs_WIRE: topods.Wire,
            TopAbs_SHELL: topods.Shell,
            TopAbs_SOLID: topods.Solid,
            TopAbs_COMPOUND: topods.Compound,
            TopAbs_COMPSOLID: topods.CompSolid
        }

    def _loop_topo(self,
                   topologyType: TopAbs_ShapeEnum,
                   topologicalEntity=None,
                   topologyTypeToAvoid=None) -> Iterator[Any]:
        '''
        this could be a faces generator for a python TopoShape class
        that way you can just do:
        for face in srf.faces:
            processFace(face)
        '''
        topoTypes = {TopAbs_VERTEX: TopoDS_Vertex,
                     TopAbs_EDGE: TopoDS_Edge,
                     TopAbs_FACE: TopoDS_Face,
                     TopAbs_WIRE: TopoDS_Wire,
                     TopAbs_SHELL: TopoDS_Shell,
                     TopAbs_SOLID: TopoDS_Solid,
                     TopAbs_COMPOUND: TopoDS_Compound,
                     TopAbs_COMPSOLID: TopoDS_CompSolid}
        topExp = TopExp_Explorer()
        if topologyType not in topoTypes.keys():
            raise AssertionError("%s not one of %s" % (topologyType, topoTypes.keys()))
        # use self.myShape if nothing is specified
        if topologicalEntity is None and topologyTypeToAvoid is None:
            topExp.Init(self.myShape, topologyType)
        elif topologicalEntity is None and topologyTypeToAvoid is not None:
            topExp.Init(self.myShape, topologyType, topologyTypeToAvoid)
        elif topologyTypeToAvoid is None:
            topExp.Init(topologicalEntity, topologyType)
        elif topologyTypeToAvoid:
            topExp.Init(topologicalEntity,
                        topologyType,
                        topologyTypeToAvoid)
        seq = []
        while topExp.More():
            current_item = topExp.Current()
            topo_to_add = self.topoFactory[topologyType](current_item)
            seq.append(topo_to_add)
            topExp.Next()

        if self.ignore_orientation:
            # filter out those entities that share the same TShape
            # but do *not* share the same orientation
            #filter_orientation_seq = []
            filter_orientation_seq: List = []
            for i in seq:
                _present = False
                for j in filter_orientation_seq:
                    if i.IsSame(j):
                        _present = True
                        break
                if _present is False:
                    filter_orientation_seq.append(i)
            return iter(filter_orientation_seq)
        else:
            return iter(seq)

    def faces(self) -> Iterator[TopoDS_Face]:
        '''
        loops over all faces
        '''
        return self._loop_topo(TopAbs_FACE)

    def _number_of_topo(self, iterable: Iterable) -> int:
        n = 0
        for _ in iterable:
            n += 1
        return n

    def number_of_faces(self) -> int:
        return self._number_of_topo(self.faces())

    def vertices(self) -> Iterator[TopoDS_Vertex]:
        '''
        loops over all vertices
        '''
        return self._loop_topo(TopAbs_VERTEX)

    def number_of_vertices(self) -> int:
        return self._number_of_topo(self.vertices())

    def edges(self) -> Iterator[TopoDS_Edge]:
        '''
        loops over all edges
        '''
        return self._loop_topo(TopAbs_EDGE)

    def number_of_edges(self) -> int:
        return self._number_of_topo(self.edges())

    def wires(self) -> Iterator[TopoDS_Wire]:
        '''
        loops over all wires
        '''
        return self._loop_topo(TopAbs_WIRE)

    def number_of_wires(self) -> int:
        return self._number_of_topo(self.wires())

    def shells(self) -> Iterator[TopoDS_Shell]:
        '''
        loops over all shells
        '''
        return self._loop_topo(TopAbs_SHELL, None)

    def number_of_shells(self) -> int:
        return self._number_of_topo(self.shells())

    def solids(self) -> Iterator[TopoDS_Solid]:
        '''
        loops over all solids
        '''
        return self._loop_topo(TopAbs_SOLID, None)

    def number_of_solids(self) -> int:
        return self._number_of_topo(self.solids())

    def comp_solids(self) -> Iterator[TopoDS_CompSolid]:
        '''
        loops over all compound solids
        '''
        return self._loop_topo(TopAbs_COMPSOLID)

    def number_of_comp_solids(self) -> int:
        return self._number_of_topo(self.comp_solids())

    def compounds(self) -> Iterator[TopoDS_Compound]:
        '''
        loops over all compounds
        '''
        return self._loop_topo(TopAbs_COMPOUND)

    def number_of_compounds(self) -> int:
        return self._number_of_topo(self.compounds())

    def ordered_vertices_from_wire(self, wire: TopoDS_Wire) -> Iterator[TopoDS_Vertex]:
        '''
        @param wire: TopoDS_Wire
        '''
        we = WireExplorer(wire)
        return we.ordered_vertices()

    def number_of_ordered_vertices_from_wire(self, wire: TopoDS_Wire) -> int:
        return self._number_of_topo(self.ordered_vertices_from_wire(wire))

    def ordered_edges_from_wire(self, wire: TopoDS_Wire) -> Iterator[TopoDS_Edge]:
        '''
        @param wire: TopoDS_Wire
        '''
        we = WireExplorer(wire)
        return we.ordered_edges()

    def number_of_ordered_edges_from_wire(self, wire: TopoDS_Wire) -> int:
        return self._number_of_topo(self.ordered_edges_from_wire(wire))

    def _map_shapes_and_ancestors(self, topoTypeA, topoTypeB, topologicalEntity):
        '''
        using the same method
        @param topoTypeA:
        @param topoTypeB:
        @param topologicalEntity:
        '''
        topo_set = set()
        _map = TopTools_IndexedDataMapOfShapeListOfShape()
        topexp_MapShapesAndAncestors(self.myShape, topoTypeA, topoTypeB, _map)
        results = _map.FindFromKey(topologicalEntity)
        if results.Size() == 0:
            yield None

        topology_iterator = TopTools_ListIteratorOfListOfShape(results)
        while topology_iterator.More():
            topo_entity = self.topoFactory[topoTypeB](topology_iterator.Value())

            # return the entity if not in set
            # to assure we're not returning entities several times
            if not topo_entity in topo_set:
                if self.ignore_orientation:
                    unique = True
                    for i in topo_set:
                        if i.IsSame(topo_entity):
                            unique = False
                            break
                    if unique:
                        yield topo_entity
                else:
                    yield topo_entity

            topo_set.add(topo_entity)
            topology_iterator.Next()

    def _number_shapes_ancestors(self, topoTypeA, topoTypeB, topologicalEntity):
        '''returns the number of shape ancestors
        If you want to know how many edges a faces has:
        _number_shapes_ancestors(self, TopAbs_EDGE, TopAbs_FACE, edg)
        will return the number of edges a faces has
        @param topoTypeA:
        @param topoTypeB:
        @param topologicalEntity:
        '''
        topo_set = set()
        _map = TopTools_IndexedDataMapOfShapeListOfShape()
        topexp_MapShapesAndAncestors(self.myShape, topoTypeA, topoTypeB, _map)
        results = _map.FindFromKey(topologicalEntity)
        if results.Size() == 0:
            return None
        topology_iterator = TopTools_ListIteratorOfListOfShape(results)
        while topology_iterator.More():
            topo_set.add(topology_iterator.Value())
            topology_iterator.Next()
        return len(topo_set)

    # ======================================================================
    # EDGE <-> FACE
    # ======================================================================
    def faces_from_edge(self, edge: TopoDS_Edge):
        """

        :param edge:
        :return:
        """
        return self._map_shapes_and_ancestors(TopAbs_EDGE, TopAbs_FACE, edge)

    def number_of_faces_from_edge(self, edge: TopoDS_Edge) -> int:
        """

        :param edge:
        :return:
        """
        return self._number_shapes_ancestors(TopAbs_EDGE, TopAbs_FACE, edge)

    def edges_from_face(self, face: TopoDS_Face) -> Iterator[TopoDS_Edge]:
        """

        :param face:
        :return:
        """
        return self._loop_topo(TopAbs_EDGE, face)

    def number_of_edges_from_face(self, face: TopoDS_Face) -> int:
        cnt = 0
        for _ in self._loop_topo(TopAbs_EDGE, face):
            cnt += 1
        return cnt

    # ======================================================================
    # VERTEX <-> EDGE
    # ======================================================================
    def vertices_from_edge(self, edge: TopoDS_Edge) -> Iterator[TopoDS_Vertex]:
        return self._loop_topo(TopAbs_VERTEX, edge)

    def number_of_vertices_from_edge(self, edge: TopoDS_Edge) -> int:
        cnt = 0
        for _ in self._loop_topo(TopAbs_VERTEX, edge):
            cnt += 1
        return cnt

    def edges_from_vertex(self, vertex):
        return self._map_shapes_and_ancestors(TopAbs_VERTEX, TopAbs_EDGE, vertex)

    def number_of_edges_from_vertex(self, vertex: TopoDS_Vertex) -> int:
        return self._number_shapes_ancestors(TopAbs_VERTEX, TopAbs_EDGE, vertex)

    # ======================================================================
    # WIRE <-> EDGE
    # ======================================================================
    def edges_from_wire(self, wire: TopoDS_Wire) -> Iterator[TopoDS_Edge]:
        return self._loop_topo(TopAbs_EDGE, wire)

    def number_of_edges_from_wire(self, wire: TopoDS_Wire) -> int:
        cnt = 0
        for _ in self._loop_topo(TopAbs_EDGE, wire):
            cnt += 1
        return cnt

    def wires_from_edge(self, edg):
        return self._map_shapes_and_ancestors(TopAbs_EDGE, TopAbs_WIRE, edg)

    def wires_from_vertex(self, edg):
        return self._map_shapes_and_ancestors(TopAbs_VERTEX, TopAbs_WIRE, edg)

    def number_of_wires_from_edge(self, edg):
        return self._number_shapes_ancestors(TopAbs_EDGE, TopAbs_WIRE, edg)

    # ======================================================================
    # WIRE <-> FACE
    # ======================================================================
    def wires_from_face(self, face: TopoDS_Face) -> Iterator[TopoDS_Wire]:
        return self._loop_topo(TopAbs_WIRE, face)

    def number_of_wires_from_face(self, face: TopoDS_Face) -> int:
        cnt = 0
        for _ in self._loop_topo(TopAbs_WIRE, face):
            cnt += 1
        return cnt

    def faces_from_wire(self, wire):
        return self._map_shapes_and_ancestors(TopAbs_WIRE, TopAbs_FACE, wire)

    def number_of_faces_from_wires(self, wire):
        return self._number_shapes_ancestors(TopAbs_WIRE, TopAbs_FACE, wire)

    # ======================================================================
    # VERTEX <-> FACE
    # ======================================================================
    def faces_from_vertex(self, vertex):
        return self._map_shapes_and_ancestors(TopAbs_VERTEX, TopAbs_FACE, vertex)

    def number_of_faces_from_vertex(self, vertex):
        return self._number_shapes_ancestors(TopAbs_VERTEX, TopAbs_FACE, vertex)

    def vertices_from_face(self, face: TopoDS_Face) -> Iterator[TopoDS_Vertex]:
        return self._loop_topo(TopAbs_VERTEX, face)

    def number_of_vertices_from_face(self, face: TopoDS_Face) -> int:
        cnt = 0
        for _ in self._loop_topo(TopAbs_VERTEX, face):
            cnt += 1
        return cnt

    # ======================================================================
    # FACE <-> SOLID
    # ======================================================================
    def solids_from_face(self, face):
        return self._map_shapes_and_ancestors(TopAbs_FACE, TopAbs_SOLID, face)

    def number_of_solids_from_face(self, face):
        return self._number_shapes_ancestors(TopAbs_FACE, TopAbs_SOLID, face)

    def faces_from_solids(self, solid: TopoDS_Solid) -> Iterator[TopoDS_Face]:
        return self._loop_topo(TopAbs_FACE, solid)

    def number_of_faces_from_solids(self, solid: TopoDS_Solid) -> int:
        cnt = 0
        for _ in self._loop_topo(TopAbs_FACE, solid):
            cnt += 1
        return cnt


def dump_topology_to_string(shape: TopoDS_Shape,
                            level: Optional[int]=0,
                            buffer: Optional[str]="") -> None:
    """
    Return the details of an object from the top down
    """
    brt = BRep_Tool()
    s = shape.ShapeType()
    if s == TopAbs_VERTEX:
        pnt = brt.Pnt(topods_Vertex(shape))
        print(".." * level  + "<Vertex %i: %s %s %s>\n" % (hash(shape), pnt.X(), pnt.Y(), pnt.Z()))
    else:
        print(".." * level, end="")
        print(shape)
    it = TopoDS_Iterator(shape)
    while it.More() and level < 5:  # LEVEL MAX
        shp = it.Value()
        it.Next()
        dump_topology_to_string(shp, level + 1, buffer)

#
# Edge and wire discretizers
#

def discretize_wire(a_topods_wire: TopoDS_Wire, deflection: Optional[int]=0.5) -> List[gp_Pnt]:
    """ Returns a set of points
    """
    if not is_wire(a_topods_wire):
        raise AssertionError("You must provide a TopoDS_Wire to the discretize_wire function.")
    wire_explorer = WireExplorer(a_topods_wire)
    wire_pnts = []
    # loop over ordered edges
    for edg in wire_explorer.ordered_edges():
        edg_pnts = discretize_edge(edg, deflection)
        wire_pnts += edg_pnts
    return wire_pnts


def discretize_edge(a_topods_edge: TopoDS_Edge, deflection=0.2, algorithm="QuasiUniformDeflection"):
    """ Take a TopoDS_Edge and returns a list of points
    The more deflection is small, the more the discretization is precise,
    i.e. the more points you get in the returned points
    algorithm: to choose in ["UniformAbscissa", "QuasiUniformDeflection"]
    """
    if not is_edge(a_topods_edge):
        raise AssertionError("You must provide a TopoDS_Edge to the discretize_edge function.")
    if a_topods_edge.IsNull():
        print("Warning : TopoDS_Edge is null. discretize_edge will return an empty list of points.")
        return []
    curve_adaptator = BRepAdaptor_Curve(a_topods_edge)
    first = curve_adaptator.FirstParameter()
    last = curve_adaptator.LastParameter()

    if algorithm == "QuasiUniformDeflection":
        discretizer = GCPnts_QuasiUniformDeflection()
    elif algorithm == "UniformAbscissa":
        discretizer = GCPnts_UniformAbscissa()
    elif algorithm == "UniformDeflection":
        discretizer = GCPnts_UniformDeflection()
    else:
        raise AssertionError("Unknown algorithm")
    discretizer.Initialize(curve_adaptator, deflection, first, last)

    if not discretizer.IsDone():
        raise AssertionError("Discretizer not done.")
    if not discretizer.NbPoints() > 0:
        raise AssertionError("Discretizer nb points not > 0.")

    points = []
    for i in range(1, discretizer.NbPoints() + 1):
        p = curve_adaptator.Value(discretizer.Parameter(i))
        points.append(p.Coord())
    return points

#
# TopoDS_Shape type utils
#
def is_vertex(topods_shape: TopoDS_Shape) -> bool:
    if not hasattr(topods_shape, "ShapeType"):
        return False
    return topods_shape.ShapeType() == TopAbs_VERTEX


def is_solid(topods_shape: TopoDS_Shape) -> bool:
    if not hasattr(topods_shape, "ShapeType"):
        return False
    return topods_shape.ShapeType() == TopAbs_SOLID


def is_edge(topods_shape: TopoDS_Shape) -> bool:
    if not hasattr(topods_shape, "ShapeType"):
        return False
    return topods_shape.ShapeType() == TopAbs_EDGE


def is_face(topods_shape: TopoDS_Shape) -> bool:
    if not hasattr(topods_shape, "ShapeType"):
        return False
    return topods_shape.ShapeType() == TopAbs_FACE


def is_shell(topods_shape: TopoDS_Shape) -> bool:
    if not hasattr(topods_shape, "ShapeType"):
        return False
    return topods_shape.ShapeType() == TopAbs_SHELL


def is_wire(topods_shape: TopoDS_Shape) -> bool:
    if not hasattr(topods_shape, "ShapeType"):
        return False
    return topods_shape.ShapeType() == TopAbs_WIRE


def is_compound(topods_shape: TopoDS_Shape) -> bool:
    if not hasattr(topods_shape, "ShapeType"):
        return False
    return topods_shape.ShapeType() == TopAbs_COMPOUND


def is_compsolid(topods_shape: TopoDS_Shape) -> bool:
    if not hasattr(topods_shape, "ShapeType"):
        return False
    return topods_shape.ShapeType() == TopAbs_COMPSOLID


def get_type_as_string(topods_shape: TopoDS_Shape) -> str:
    """ just get the type string, remove TopAbs_ and lowercas all ending letters
    """
    types = {TopAbs_VERTEX: "Vertex", TopAbs_COMPSOLID: "CompSolid", TopAbs_FACE: "Face",
             TopAbs_WIRE: "Wire", TopAbs_EDGE: "Edge", TopAbs_COMPOUND: "Compound",
             TopAbs_COMPSOLID: "CompSolid", TopAbs_SOLID: "Solid"}
    return types[topods_shape.ShapeType()]


def get_sorted_hlr_edges(topods_shape: TopoDS_Shape,
                         position: Optional[gp_Pnt] =gp_Pnt(),
                         direction: Optional[gp_Dir] = gp_Dir(),
                         export_hidden_edges: Optional[bool] =True) -> Tuple[List, List]:
    """ Return hidden and visible edges as two lists of edges
    """
    hlr = HLRBRep_Algo()
    hlr.Add(topods_shape)

    projector = HLRAlgo_Projector(gp_Ax2(position, direction))

    hlr.Projector(projector)
    hlr.Update()
    hlr.Hide()

    hlr_shapes = HLRBRep_HLRToShape(hlr)

    # visible edges
    visible = []
    visible_sharp_edges_as_compound = hlr_shapes.VCompound()
    if visible_sharp_edges_as_compound:
        visible += list(TopologyExplorer(visible_sharp_edges_as_compound).edges())
    visible_smooth_edges_as_compound = hlr_shapes.Rg1LineVCompound()
    if visible_smooth_edges_as_compound:
        visible += list(TopologyExplorer(visible_smooth_edges_as_compound).edges())
    #visible_sewn_edges_as_compound = hlr_shapes.RgNLineVCompound()
    #if visible_sewn_edges_as_compound:
    #    visible += list(TopologyExplorer(visible_sewn_edges_as_compound).edges())
    visible_contour_edges_as_compound = hlr_shapes.OutLineVCompound()
    if visible_contour_edges_as_compound:
        visible += list(TopologyExplorer(visible_contour_edges_as_compound).edges())
    #visible_isoparameter_edges_as_compound = hlr_shapes.IsoLineVCompound()
    #if visible_isoparameter_edges_as_compound:
    #    visible += list(TopologyExplorer(visible_isoparameter_edges_as_compound).edges())
    # hidden edges
    hidden = []
    if export_hidden_edges:
        hidden_sharp_edges_as_compound = hlr_shapes.HCompound()
        if hidden_sharp_edges_as_compound:
            hidden += list(TopologyExplorer(hidden_sharp_edges_as_compound).edges())
        hidden_contour_edges_as_compound = hlr_shapes.OutLineHCompound()
        if hidden_contour_edges_as_compound:
            hidden += list(TopologyExplorer(hidden_contour_edges_as_compound).edges())

    return visible, hidden


def list_of_shapes_to_compound(list_of_shapes: List[TopoDS_Shape]) -> Tuple[TopoDS_Compound, bool]:
    """ takes a list of shape in input, gather all shapes into one compound
    returns the compund and a boolean, True if all shapes were added to the compund,
    False otherwise
    """
    all_shapes_converted = True
    the_compound = TopoDS_Compound()
    the_builder = BRep_Builder()
    the_builder.MakeCompound(the_compound)
    for shp in list_of_shapes:
        # first ensure the shape is not Null
        if shp.IsNull():
            all_shapes_converted = False
            continue
        else:
            the_builder.Add(the_compound, shp)
    return the_compound, all_shapes_converted



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""
GeomAdaptor module, see official documentation at
https://www.opencascade.com/doc/occt-7.4.0/refman/html/package_geomadaptor.html
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _GeomAdaptor
else:
    import _GeomAdaptor

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _GeomAdaptor.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _GeomAdaptor.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _GeomAdaptor.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _GeomAdaptor.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _GeomAdaptor.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _GeomAdaptor.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _GeomAdaptor.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _GeomAdaptor.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _GeomAdaptor.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _GeomAdaptor.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _GeomAdaptor.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _GeomAdaptor.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _GeomAdaptor.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _GeomAdaptor.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _GeomAdaptor.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _GeomAdaptor.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _GeomAdaptor.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _GeomAdaptor:
_GeomAdaptor.SwigPyIterator_swigregister(SwigPyIterator)


def _dumps_object(klass):
    """ Overwrite default string output for any wrapped object.
    By default, __repr__ method returns something like:
    <OCC.Core.TopoDS.TopoDS_Shape; proxy of <Swig Object of type 'TopoDS_Shape *' at 0x02BB0758> >
    This is too much verbose.
    We prefer :
    <class 'gp_Pnt'>
    or
    <class 'TopoDS_Shape'>
    """
    klass_name = str(klass.__class__).split(".")[3].split("'")[0]
    repr_string = "<class '" + klass_name + "'"
# for TopoDS_Shape, we also look for the base type
    if klass_name == "TopoDS_Shape":
        if klass.IsNull():
            repr_string += ": Null>"
            return repr_string
        st = klass.ShapeType()
        types = {OCC.Core.TopAbs.TopAbs_VERTEX: "Vertex",
                 OCC.Core.TopAbs.TopAbs_SOLID: "Solid",
                 OCC.Core.TopAbs.TopAbs_EDGE: "Edge",
                 OCC.Core.TopAbs.TopAbs_FACE: "Face",
                 OCC.Core.TopAbs.TopAbs_SHELL: "Shell",
                 OCC.Core.TopAbs.TopAbs_WIRE: "Wire",
                 OCC.Core.TopAbs.TopAbs_COMPOUND: "Compound",
                 OCC.Core.TopAbs.TopAbs_COMPSOLID: "Compsolid"}
        repr_string += "; Type:%s" % types[st]        
    elif hasattr(klass, "IsNull"):
        if klass.IsNull():
            repr_string += "; Null"
    repr_string += ">"
    return repr_string


def process_exception(error: "Standard_Failure", method_name: "std::string", class_name: "std::string") -> "void":
    return _GeomAdaptor.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

import OCC.Core.Standard
import OCC.Core.NCollection
import OCC.Core.Adaptor3d
import OCC.Core.Geom
import OCC.Core.gp
import OCC.Core.TColStd
import OCC.Core.TCollection
import OCC.Core.GeomAbs
import OCC.Core.TColgp
import OCC.Core.TopAbs
import OCC.Core.Adaptor2d
import OCC.Core.Geom2d
import OCC.Core.math
import OCC.Core.Message
import OCC.Core.OSD

from enum import IntEnum
from OCC.Core.Exception import *




def Handle_GeomAdaptor_GHCurve_Create() -> "opencascade::handle< GeomAdaptor_GHCurve >":
    return _GeomAdaptor.Handle_GeomAdaptor_GHCurve_Create()

def Handle_GeomAdaptor_GHCurve_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< GeomAdaptor_GHCurve >":
    return _GeomAdaptor.Handle_GeomAdaptor_GHCurve_DownCast(t)

def Handle_GeomAdaptor_GHCurve_IsNull(t: "opencascade::handle< GeomAdaptor_GHCurve > const &") -> "bool":
    return _GeomAdaptor.Handle_GeomAdaptor_GHCurve_IsNull(t)

def Handle_GeomAdaptor_GHSurface_Create() -> "opencascade::handle< GeomAdaptor_GHSurface >":
    return _GeomAdaptor.Handle_GeomAdaptor_GHSurface_Create()

def Handle_GeomAdaptor_GHSurface_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< GeomAdaptor_GHSurface >":
    return _GeomAdaptor.Handle_GeomAdaptor_GHSurface_DownCast(t)

def Handle_GeomAdaptor_GHSurface_IsNull(t: "opencascade::handle< GeomAdaptor_GHSurface > const &") -> "bool":
    return _GeomAdaptor.Handle_GeomAdaptor_GHSurface_IsNull(t)

def Handle_GeomAdaptor_HSurfaceOfLinearExtrusion_Create() -> "opencascade::handle< GeomAdaptor_HSurfaceOfLinearExtrusion >":
    return _GeomAdaptor.Handle_GeomAdaptor_HSurfaceOfLinearExtrusion_Create()

def Handle_GeomAdaptor_HSurfaceOfLinearExtrusion_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< GeomAdaptor_HSurfaceOfLinearExtrusion >":
    return _GeomAdaptor.Handle_GeomAdaptor_HSurfaceOfLinearExtrusion_DownCast(t)

def Handle_GeomAdaptor_HSurfaceOfLinearExtrusion_IsNull(t: "opencascade::handle< GeomAdaptor_HSurfaceOfLinearExtrusion > const &") -> "bool":
    return _GeomAdaptor.Handle_GeomAdaptor_HSurfaceOfLinearExtrusion_IsNull(t)

def Handle_GeomAdaptor_HSurfaceOfRevolution_Create() -> "opencascade::handle< GeomAdaptor_HSurfaceOfRevolution >":
    return _GeomAdaptor.Handle_GeomAdaptor_HSurfaceOfRevolution_Create()

def Handle_GeomAdaptor_HSurfaceOfRevolution_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< GeomAdaptor_HSurfaceOfRevolution >":
    return _GeomAdaptor.Handle_GeomAdaptor_HSurfaceOfRevolution_DownCast(t)

def Handle_GeomAdaptor_HSurfaceOfRevolution_IsNull(t: "opencascade::handle< GeomAdaptor_HSurfaceOfRevolution > const &") -> "bool":
    return _GeomAdaptor.Handle_GeomAdaptor_HSurfaceOfRevolution_IsNull(t)

def Handle_GeomAdaptor_HCurve_Create() -> "opencascade::handle< GeomAdaptor_HCurve >":
    return _GeomAdaptor.Handle_GeomAdaptor_HCurve_Create()

def Handle_GeomAdaptor_HCurve_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< GeomAdaptor_HCurve >":
    return _GeomAdaptor.Handle_GeomAdaptor_HCurve_DownCast(t)

def Handle_GeomAdaptor_HCurve_IsNull(t: "opencascade::handle< GeomAdaptor_HCurve > const &") -> "bool":
    return _GeomAdaptor.Handle_GeomAdaptor_HCurve_IsNull(t)

def Handle_GeomAdaptor_HSurface_Create() -> "opencascade::handle< GeomAdaptor_HSurface >":
    return _GeomAdaptor.Handle_GeomAdaptor_HSurface_Create()

def Handle_GeomAdaptor_HSurface_DownCast(t: "opencascade::handle< Standard_Transient > const &") -> "opencascade::handle< GeomAdaptor_HSurface >":
    return _GeomAdaptor.Handle_GeomAdaptor_HSurface_DownCast(t)

def Handle_GeomAdaptor_HSurface_IsNull(t: "opencascade::handle< GeomAdaptor_HSurface > const &") -> "bool":
    return _GeomAdaptor.Handle_GeomAdaptor_HSurface_IsNull(t)
class geomadaptor(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    @staticmethod
    def MakeCurve(*args) -> "opencascade::handle< Geom_Curve >":
        r"""
        Inherited from ghcurve. provides a curve handled by reference. build a geom_curve using the informations from the curve from adaptor3d.

        Parameters
        ----------
        C: Adaptor3d_Curve

        Returns
        -------
        opencascade::handle<Geom_Curve>

        """
        return _GeomAdaptor.geomadaptor_MakeCurve(*args)

    @staticmethod
    def MakeSurface(*args) -> "opencascade::handle< Geom_Surface >":
        r"""
        Build a geom_surface using the informations from the surface from adaptor3d @param thes - surface adaptor to convert. @param thetrimflag - true if perform trim surface values by adaptor and false otherwise.

        Parameters
        ----------
        theS: Adaptor3d_Surface
        theTrimFlag: bool,optional
        	default value is Standard_True

        Returns
        -------
        opencascade::handle<Geom_Surface>

        """
        return _GeomAdaptor.geomadaptor_MakeSurface(*args)

    __repr__ = _dumps_object


    def __init__(self):
        _GeomAdaptor.geomadaptor_swiginit(self, _GeomAdaptor.new_geomadaptor())
    __swig_destroy__ = _GeomAdaptor.delete_geomadaptor

# Register geomadaptor in _GeomAdaptor:
_GeomAdaptor.geomadaptor_swigregister(geomadaptor)

def geomadaptor_MakeCurve(*args) -> "opencascade::handle< Geom_Curve >":
    r"""
    Inherited from ghcurve. provides a curve handled by reference. build a geom_curve using the informations from the curve from adaptor3d.

    Parameters
    ----------
    C: Adaptor3d_Curve

    Returns
    -------
    opencascade::handle<Geom_Curve>

    """
    return _GeomAdaptor.geomadaptor_MakeCurve(*args)

def geomadaptor_MakeSurface(*args) -> "opencascade::handle< Geom_Surface >":
    r"""
    Build a geom_surface using the informations from the surface from adaptor3d @param thes - surface adaptor to convert. @param thetrimflag - true if perform trim surface values by adaptor and false otherwise.

    Parameters
    ----------
    theS: Adaptor3d_Surface
    theTrimFlag: bool,optional
    	default value is Standard_True

    Returns
    -------
    opencascade::handle<Geom_Surface>

    """
    return _GeomAdaptor.geomadaptor_MakeSurface(*args)

class GeomAdaptor_Curve(OCC.Core.Adaptor3d.Adaptor3d_Curve):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Returns
        -------
        None
        No available documentation.

        Parameters
        ----------
        C: Geom_Curve

        Returns
        -------
        None
        Constructionerror is raised if ufirst>ulast.

        Parameters
        ----------
        C: Geom_Curve
        UFirst: float
        ULast: float

        Returns
        -------
        None

        """
        _GeomAdaptor.GeomAdaptor_Curve_swiginit(self, _GeomAdaptor.new_GeomAdaptor_Curve(*args))

    def Curve(self, *args) -> "opencascade::handle< Geom_Curve > const &":
        r"""
        Provides a curve inherited from hcurve from adaptor. this is inherited to provide easy to use constructors.

        Returns
        -------
        opencascade::handle<Geom_Curve>

        """
        return _GeomAdaptor.GeomAdaptor_Curve_Curve(self, *args)

    def Load(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        C: Geom_Curve

        Returns
        -------
        None
        Constructionerror is raised if ufirst>ulast.

        Parameters
        ----------
        C: Geom_Curve
        UFirst: float
        ULast: float

        Returns
        -------
        None

        """
        return _GeomAdaptor.GeomAdaptor_Curve_Load(self, *args)

    def Reset(self, *args) -> "void":
        r"""
        Reset currently loaded curve (undone load()).

        Returns
        -------
        None

        """
        return _GeomAdaptor.GeomAdaptor_Curve_Reset(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _GeomAdaptor.delete_GeomAdaptor_Curve

# Register GeomAdaptor_Curve in _GeomAdaptor:
_GeomAdaptor.GeomAdaptor_Curve_swigregister(GeomAdaptor_Curve)

class GeomAdaptor_GHCurve(OCC.Core.Adaptor3d.Adaptor3d_HCurve):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates an empty genhcurve.

        Returns
        -------
        None
        Creates a genhcurve from a curve.

        Parameters
        ----------
        C: GeomAdaptor_Curve

        Returns
        -------
        None

        """
        _GeomAdaptor.GeomAdaptor_GHCurve_swiginit(self, _GeomAdaptor.new_GeomAdaptor_GHCurve(*args))

    def ChangeCurve(self, *args) -> "GeomAdaptor_Curve &":
        r"""
        Returns the curve used to create the genhcurve.

        Returns
        -------
        GeomAdaptor_Curve

        """
        return _GeomAdaptor.GeomAdaptor_GHCurve_ChangeCurve(self, *args)

    def Set(self, *args) -> "void":
        r"""
        Sets the field of the genhcurve.

        Parameters
        ----------
        C: GeomAdaptor_Curve

        Returns
        -------
        None

        """
        return _GeomAdaptor.GeomAdaptor_GHCurve_Set(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_GeomAdaptor_GHCurve_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _GeomAdaptor.delete_GeomAdaptor_GHCurve

# Register GeomAdaptor_GHCurve in _GeomAdaptor:
_GeomAdaptor.GeomAdaptor_GHCurve_swigregister(GeomAdaptor_GHCurve)

class GeomAdaptor_GHSurface(OCC.Core.Adaptor3d.Adaptor3d_HSurface):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates an empty genhsurface.

        Returns
        -------
        None
        Creates a genhsurface from a surface.

        Parameters
        ----------
        S: GeomAdaptor_Surface

        Returns
        -------
        None

        """
        _GeomAdaptor.GeomAdaptor_GHSurface_swiginit(self, _GeomAdaptor.new_GeomAdaptor_GHSurface(*args))

    def ChangeSurface(self, *args) -> "GeomAdaptor_Surface &":
        r"""
        Returns the surface used to create the genhsurface.

        Returns
        -------
        GeomAdaptor_Surface

        """
        return _GeomAdaptor.GeomAdaptor_GHSurface_ChangeSurface(self, *args)

    def Set(self, *args) -> "void":
        r"""
        Sets the field of the genhsurface.

        Parameters
        ----------
        S: GeomAdaptor_Surface

        Returns
        -------
        None

        """
        return _GeomAdaptor.GeomAdaptor_GHSurface_Set(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_GeomAdaptor_GHSurface_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _GeomAdaptor.delete_GeomAdaptor_GHSurface

# Register GeomAdaptor_GHSurface in _GeomAdaptor:
_GeomAdaptor.GeomAdaptor_GHSurface_swigregister(GeomAdaptor_GHSurface)

class GeomAdaptor_HSurfaceOfLinearExtrusion(OCC.Core.Adaptor3d.Adaptor3d_HSurface):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates an empty genhsurface.

        Returns
        -------
        None
        Creates a genhsurface from a surface.

        Parameters
        ----------
        S: GeomAdaptor_SurfaceOfLinearExtrusion

        Returns
        -------
        None

        """
        _GeomAdaptor.GeomAdaptor_HSurfaceOfLinearExtrusion_swiginit(self, _GeomAdaptor.new_GeomAdaptor_HSurfaceOfLinearExtrusion(*args))

    def ChangeSurface(self, *args) -> "GeomAdaptor_SurfaceOfLinearExtrusion &":
        r"""
        Returns the surface used to create the genhsurface.

        Returns
        -------
        GeomAdaptor_SurfaceOfLinearExtrusion

        """
        return _GeomAdaptor.GeomAdaptor_HSurfaceOfLinearExtrusion_ChangeSurface(self, *args)

    def Set(self, *args) -> "void":
        r"""
        Sets the field of the genhsurface.

        Parameters
        ----------
        S: GeomAdaptor_SurfaceOfLinearExtrusion

        Returns
        -------
        None

        """
        return _GeomAdaptor.GeomAdaptor_HSurfaceOfLinearExtrusion_Set(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_GeomAdaptor_HSurfaceOfLinearExtrusion_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _GeomAdaptor.delete_GeomAdaptor_HSurfaceOfLinearExtrusion

# Register GeomAdaptor_HSurfaceOfLinearExtrusion in _GeomAdaptor:
_GeomAdaptor.GeomAdaptor_HSurfaceOfLinearExtrusion_swigregister(GeomAdaptor_HSurfaceOfLinearExtrusion)

class GeomAdaptor_HSurfaceOfRevolution(OCC.Core.Adaptor3d.Adaptor3d_HSurface):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates an empty genhsurface.

        Returns
        -------
        None
        Creates a genhsurface from a surface.

        Parameters
        ----------
        S: GeomAdaptor_SurfaceOfRevolution

        Returns
        -------
        None

        """
        _GeomAdaptor.GeomAdaptor_HSurfaceOfRevolution_swiginit(self, _GeomAdaptor.new_GeomAdaptor_HSurfaceOfRevolution(*args))

    def ChangeSurface(self, *args) -> "GeomAdaptor_SurfaceOfRevolution &":
        r"""
        Returns the surface used to create the genhsurface.

        Returns
        -------
        GeomAdaptor_SurfaceOfRevolution

        """
        return _GeomAdaptor.GeomAdaptor_HSurfaceOfRevolution_ChangeSurface(self, *args)

    def Set(self, *args) -> "void":
        r"""
        Sets the field of the genhsurface.

        Parameters
        ----------
        S: GeomAdaptor_SurfaceOfRevolution

        Returns
        -------
        None

        """
        return _GeomAdaptor.GeomAdaptor_HSurfaceOfRevolution_Set(self, *args)


    @staticmethod
    def DownCast(t):
      return Handle_GeomAdaptor_HSurfaceOfRevolution_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _GeomAdaptor.delete_GeomAdaptor_HSurfaceOfRevolution

# Register GeomAdaptor_HSurfaceOfRevolution in _GeomAdaptor:
_GeomAdaptor.GeomAdaptor_HSurfaceOfRevolution_swigregister(GeomAdaptor_HSurfaceOfRevolution)

class GeomAdaptor_Surface(OCC.Core.Adaptor3d.Adaptor3d_Surface):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Returns
        -------
        None
        No available documentation.

        Parameters
        ----------
        S: Geom_Surface

        Returns
        -------
        None
        Constructionerror is raised if ufirst>ulast or vfirst>vlast.

        Parameters
        ----------
        S: Geom_Surface
        UFirst: float
        ULast: float
        VFirst: float
        VLast: float
        TolU: float,optional
        	default value is 0.0
        TolV: float,optional
        	default value is 0.0

        Returns
        -------
        None

        """
        _GeomAdaptor.GeomAdaptor_Surface_swiginit(self, _GeomAdaptor.new_GeomAdaptor_Surface(*args))

    def Load(self, *args) -> "void":
        r"""
        No available documentation.

        Parameters
        ----------
        S: Geom_Surface

        Returns
        -------
        None
        Constructionerror is raised if ufirst>ulast or vfirst>vlast.

        Parameters
        ----------
        S: Geom_Surface
        UFirst: float
        ULast: float
        VFirst: float
        VLast: float
        TolU: float,optional
        	default value is 0.0
        TolV: float,optional
        	default value is 0.0

        Returns
        -------
        None

        """
        return _GeomAdaptor.GeomAdaptor_Surface_Load(self, *args)

    def Surface(self, *args) -> "opencascade::handle< Geom_Surface > const &":
        r"""
        No available documentation.

        Returns
        -------
        opencascade::handle<Geom_Surface>

        """
        return _GeomAdaptor.GeomAdaptor_Surface_Surface(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _GeomAdaptor.delete_GeomAdaptor_Surface

# Register GeomAdaptor_Surface in _GeomAdaptor:
_GeomAdaptor.GeomAdaptor_Surface_swigregister(GeomAdaptor_Surface)

class GeomAdaptor_HCurve(GeomAdaptor_GHCurve):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Returns
        -------
        None
        No available documentation.

        Parameters
        ----------
        AS: GeomAdaptor_Curve

        Returns
        -------
        None
        No available documentation.

        Parameters
        ----------
        S: Geom_Curve

        Returns
        -------
        None
        Constructionerror is raised if ufirst>ulast or vfirst>vlast.

        Parameters
        ----------
        S: Geom_Curve
        UFirst: float
        ULast: float

        Returns
        -------
        None

        """
        _GeomAdaptor.GeomAdaptor_HCurve_swiginit(self, _GeomAdaptor.new_GeomAdaptor_HCurve(*args))


    @staticmethod
    def DownCast(t):
      return Handle_GeomAdaptor_HCurve_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _GeomAdaptor.delete_GeomAdaptor_HCurve

# Register GeomAdaptor_HCurve in _GeomAdaptor:
_GeomAdaptor.GeomAdaptor_HCurve_swigregister(GeomAdaptor_HCurve)

class GeomAdaptor_HSurface(GeomAdaptor_GHSurface):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Returns
        -------
        None
        No available documentation.

        Parameters
        ----------
        AS: GeomAdaptor_Surface

        Returns
        -------
        None
        No available documentation.

        Parameters
        ----------
        S: Geom_Surface

        Returns
        -------
        None
        Constructionerror is raised if ufirst>ulast or vfirst>vlast.

        Parameters
        ----------
        S: Geom_Surface
        UFirst: float
        ULast: float
        VFirst: float
        VLast: float
        TolU: float,optional
        	default value is 0.0
        TolV: float,optional
        	default value is 0.0

        Returns
        -------
        None

        """
        _GeomAdaptor.GeomAdaptor_HSurface_swiginit(self, _GeomAdaptor.new_GeomAdaptor_HSurface(*args))


    @staticmethod
    def DownCast(t):
      return Handle_GeomAdaptor_HSurface_DownCast(t)


    __repr__ = _dumps_object

    __swig_destroy__ = _GeomAdaptor.delete_GeomAdaptor_HSurface

# Register GeomAdaptor_HSurface in _GeomAdaptor:
_GeomAdaptor.GeomAdaptor_HSurface_swigregister(GeomAdaptor_HSurface)

class GeomAdaptor_SurfaceOfLinearExtrusion(GeomAdaptor_Surface):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Returns
        -------
        None
        The curve is loaded.

        Parameters
        ----------
        C: Adaptor3d_HCurve

        Returns
        -------
        None
        Thew curve and the direction are loaded.

        Parameters
        ----------
        C: Adaptor3d_HCurve
        V: gp_Dir

        Returns
        -------
        None

        """
        _GeomAdaptor.GeomAdaptor_SurfaceOfLinearExtrusion_swiginit(self, _GeomAdaptor.new_GeomAdaptor_SurfaceOfLinearExtrusion(*args))

    def Load(self, *args) -> "void":
        r"""
        Changes the curve.

        Parameters
        ----------
        C: Adaptor3d_HCurve

        Returns
        -------
        None
        Changes the direction.

        Parameters
        ----------
        V: gp_Dir

        Returns
        -------
        None

        """
        return _GeomAdaptor.GeomAdaptor_SurfaceOfLinearExtrusion_Load(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _GeomAdaptor.delete_GeomAdaptor_SurfaceOfLinearExtrusion

# Register GeomAdaptor_SurfaceOfLinearExtrusion in _GeomAdaptor:
_GeomAdaptor.GeomAdaptor_SurfaceOfLinearExtrusion_swigregister(GeomAdaptor_SurfaceOfLinearExtrusion)

class GeomAdaptor_SurfaceOfRevolution(GeomAdaptor_Surface):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        No available documentation.

        Returns
        -------
        None
        The curve is loaded.

        Parameters
        ----------
        C: Adaptor3d_HCurve

        Returns
        -------
        None
        The curve and the direction are loaded.

        Parameters
        ----------
        C: Adaptor3d_HCurve
        V: gp_Ax1

        Returns
        -------
        None

        """
        _GeomAdaptor.GeomAdaptor_SurfaceOfRevolution_swiginit(self, _GeomAdaptor.new_GeomAdaptor_SurfaceOfRevolution(*args))

    def Axis(self, *args) -> "gp_Ax3 const":
        r"""
        No available documentation.

        Returns
        -------
        gp_Ax3

        """
        return _GeomAdaptor.GeomAdaptor_SurfaceOfRevolution_Axis(self, *args)

    def Load(self, *args) -> "void":
        r"""
        Changes the curve.

        Parameters
        ----------
        C: Adaptor3d_HCurve

        Returns
        -------
        None
        Changes the direction.

        Parameters
        ----------
        V: gp_Ax1

        Returns
        -------
        None

        """
        return _GeomAdaptor.GeomAdaptor_SurfaceOfRevolution_Load(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _GeomAdaptor.delete_GeomAdaptor_SurfaceOfRevolution

# Register GeomAdaptor_SurfaceOfRevolution in _GeomAdaptor:
_GeomAdaptor.GeomAdaptor_SurfaceOfRevolution_swigregister(GeomAdaptor_SurfaceOfRevolution)








from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *

FSD_BStream = NewType('FSD_BStream', FILE)
#the following typedef cannot be wrapped as is
FSD_FStream = NewType('FSD_FStream', Any)

#classnotwrapped
class FSD_Base64Decoder: ...

#classnotwrapped
class FSD_File: ...

#classnotwrapped
class FSD_BinaryFile: ...

#classnotwrapped
class FSD_CmpFile: ...

#classnotwrapped
class FSD_FileHeader: ...

# harray1 classes
# harray2 classes
# hsequence classes




# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""
HatchGen module, see official documentation at
https://www.opencascade.com/doc/occt-7.4.0/refman/html/package_hatchgen.html
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _HatchGen
else:
    import _HatchGen

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _HatchGen.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _HatchGen.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _HatchGen.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _HatchGen.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _HatchGen.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _HatchGen.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _HatchGen.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _HatchGen.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _HatchGen.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _HatchGen.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _HatchGen.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _HatchGen.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _HatchGen.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _HatchGen.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _HatchGen.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _HatchGen.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _HatchGen.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _HatchGen:
_HatchGen.SwigPyIterator_swigregister(SwigPyIterator)


def _dumps_object(klass):
    """ Overwrite default string output for any wrapped object.
    By default, __repr__ method returns something like:
    <OCC.Core.TopoDS.TopoDS_Shape; proxy of <Swig Object of type 'TopoDS_Shape *' at 0x02BB0758> >
    This is too much verbose.
    We prefer :
    <class 'gp_Pnt'>
    or
    <class 'TopoDS_Shape'>
    """
    klass_name = str(klass.__class__).split(".")[3].split("'")[0]
    repr_string = "<class '" + klass_name + "'"
# for TopoDS_Shape, we also look for the base type
    if klass_name == "TopoDS_Shape":
        if klass.IsNull():
            repr_string += ": Null>"
            return repr_string
        st = klass.ShapeType()
        types = {OCC.Core.TopAbs.TopAbs_VERTEX: "Vertex",
                 OCC.Core.TopAbs.TopAbs_SOLID: "Solid",
                 OCC.Core.TopAbs.TopAbs_EDGE: "Edge",
                 OCC.Core.TopAbs.TopAbs_FACE: "Face",
                 OCC.Core.TopAbs.TopAbs_SHELL: "Shell",
                 OCC.Core.TopAbs.TopAbs_WIRE: "Wire",
                 OCC.Core.TopAbs.TopAbs_COMPOUND: "Compound",
                 OCC.Core.TopAbs.TopAbs_COMPSOLID: "Compsolid"}
        repr_string += "; Type:%s" % types[st]        
    elif hasattr(klass, "IsNull"):
        if klass.IsNull():
            repr_string += "; Null"
    repr_string += ">"
    return repr_string


def process_exception(error: "Standard_Failure", method_name: "std::string", class_name: "std::string") -> "void":
    return _HatchGen.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

import OCC.Core.Standard
import OCC.Core.NCollection
import OCC.Core.TopAbs
import OCC.Core.IntRes2d
import OCC.Core.gp
import OCC.Core.TColStd
import OCC.Core.TCollection

from enum import IntEnum
from OCC.Core.Exception import *

HatchGen_NoProblem = _HatchGen.HatchGen_NoProblem
HatchGen_TrimFailure = _HatchGen.HatchGen_TrimFailure
HatchGen_TransitionFailure = _HatchGen.HatchGen_TransitionFailure
HatchGen_IncoherentParity = _HatchGen.HatchGen_IncoherentParity
HatchGen_IncompatibleStates = _HatchGen.HatchGen_IncompatibleStates
HatchGen_TRUE = _HatchGen.HatchGen_TRUE
HatchGen_TOUCH = _HatchGen.HatchGen_TOUCH
HatchGen_TANGENT = _HatchGen.HatchGen_TANGENT
HatchGen_UNDETERMINED = _HatchGen.HatchGen_UNDETERMINED


class HatchGen_ErrorStatus(IntEnum):
	HatchGen_NoProblem = 0
	HatchGen_TrimFailure = 1
	HatchGen_TransitionFailure = 2
	HatchGen_IncoherentParity = 3
	HatchGen_IncompatibleStates = 4
HatchGen_NoProblem = HatchGen_ErrorStatus.HatchGen_NoProblem
HatchGen_TrimFailure = HatchGen_ErrorStatus.HatchGen_TrimFailure
HatchGen_TransitionFailure = HatchGen_ErrorStatus.HatchGen_TransitionFailure
HatchGen_IncoherentParity = HatchGen_ErrorStatus.HatchGen_IncoherentParity
HatchGen_IncompatibleStates = HatchGen_ErrorStatus.HatchGen_IncompatibleStates

class HatchGen_IntersectionType(IntEnum):
	HatchGen_TRUE = 0
	HatchGen_TOUCH = 1
	HatchGen_TANGENT = 2
	HatchGen_UNDETERMINED = 3
HatchGen_TRUE = HatchGen_IntersectionType.HatchGen_TRUE
HatchGen_TOUCH = HatchGen_IntersectionType.HatchGen_TOUCH
HatchGen_TANGENT = HatchGen_IntersectionType.HatchGen_TANGENT
HatchGen_UNDETERMINED = HatchGen_IntersectionType.HatchGen_UNDETERMINED

class HatchGen_Domains(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def begin(self) -> "NCollection_Sequence< HatchGen_Domain >::iterator":
        return _HatchGen.HatchGen_Domains_begin(self)

    def end(self) -> "NCollection_Sequence< HatchGen_Domain >::iterator":
        return _HatchGen.HatchGen_Domains_end(self)

    def cbegin(self) -> "NCollection_Sequence< HatchGen_Domain >::const_iterator":
        return _HatchGen.HatchGen_Domains_cbegin(self)

    def cend(self) -> "NCollection_Sequence< HatchGen_Domain >::const_iterator":
        return _HatchGen.HatchGen_Domains_cend(self)

    def __init__(self, *args):
        _HatchGen.HatchGen_Domains_swiginit(self, _HatchGen.new_HatchGen_Domains(*args))

    def Size(self) -> "Standard_Integer":
        return _HatchGen.HatchGen_Domains_Size(self)

    def Length(self) -> "Standard_Integer":
        return _HatchGen.HatchGen_Domains_Length(self)

    def Lower(self) -> "Standard_Integer":
        return _HatchGen.HatchGen_Domains_Lower(self)

    def Upper(self) -> "Standard_Integer":
        return _HatchGen.HatchGen_Domains_Upper(self)

    def IsEmpty(self) -> "Standard_Boolean":
        return _HatchGen.HatchGen_Domains_IsEmpty(self)

    def Reverse(self) -> "void":
        return _HatchGen.HatchGen_Domains_Reverse(self)

    def Exchange(self, I: "Standard_Integer const", J: "Standard_Integer const") -> "void":
        return _HatchGen.HatchGen_Domains_Exchange(self, I, J)

    @staticmethod
    def delNode(theNode: "NCollection_SeqNode *", theAl: "opencascade::handle< NCollection_BaseAllocator > &") -> "void":
        return _HatchGen.HatchGen_Domains_delNode(theNode, theAl)

    def Clear(self, theAllocator: "opencascade::handle< NCollection_BaseAllocator > const &"=0) -> "void":
        return _HatchGen.HatchGen_Domains_Clear(self, theAllocator)

    def Assign(self, theOther: "HatchGen_Domains") -> "NCollection_Sequence< HatchGen_Domain > &":
        return _HatchGen.HatchGen_Domains_Assign(self, theOther)

    def Set(self, theOther: "HatchGen_Domains") -> "NCollection_Sequence< HatchGen_Domain > &":
        return _HatchGen.HatchGen_Domains_Set(self, theOther)

    def Remove(self, *args) -> "void":
        return _HatchGen.HatchGen_Domains_Remove(self, *args)

    def Append(self, *args) -> "void":
        return _HatchGen.HatchGen_Domains_Append(self, *args)

    def Prepend(self, *args) -> "void":
        return _HatchGen.HatchGen_Domains_Prepend(self, *args)

    def InsertBefore(self, *args) -> "void":
        return _HatchGen.HatchGen_Domains_InsertBefore(self, *args)

    def InsertAfter(self, *args) -> "void":
        return _HatchGen.HatchGen_Domains_InsertAfter(self, *args)

    def Split(self, theIndex: "Standard_Integer const", theSeq: "HatchGen_Domains") -> "void":
        return _HatchGen.HatchGen_Domains_Split(self, theIndex, theSeq)

    def First(self) -> "HatchGen_Domain const &":
        return _HatchGen.HatchGen_Domains_First(self)

    def ChangeFirst(self) -> "HatchGen_Domain &":
        return _HatchGen.HatchGen_Domains_ChangeFirst(self)

    def Last(self) -> "HatchGen_Domain const &":
        return _HatchGen.HatchGen_Domains_Last(self)

    def ChangeLast(self) -> "HatchGen_Domain &":
        return _HatchGen.HatchGen_Domains_ChangeLast(self)

    def Value(self, theIndex: "Standard_Integer const") -> "HatchGen_Domain const &":
        return _HatchGen.HatchGen_Domains_Value(self, theIndex)

    def ChangeValue(self, theIndex: "Standard_Integer const") -> "HatchGen_Domain &":
        return _HatchGen.HatchGen_Domains_ChangeValue(self, theIndex)

    def __call__(self, *args) -> "HatchGen_Domain &":
        return _HatchGen.HatchGen_Domains___call__(self, *args)

    def SetValue(self, theIndex: "Standard_Integer const", theItem: "HatchGen_Domain") -> "void":
        return _HatchGen.HatchGen_Domains_SetValue(self, theIndex, theItem)
    __swig_destroy__ = _HatchGen.delete_HatchGen_Domains

    def __len__(self):
        return self.Size()


# Register HatchGen_Domains in _HatchGen:
_HatchGen.HatchGen_Domains_swigregister(HatchGen_Domains)

def HatchGen_Domains_delNode(theNode: "NCollection_SeqNode *", theAl: "opencascade::handle< NCollection_BaseAllocator > &") -> "void":
    return _HatchGen.HatchGen_Domains_delNode(theNode, theAl)

class HatchGen_PointsOnElement(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def begin(self) -> "NCollection_Sequence< HatchGen_PointOnElement >::iterator":
        return _HatchGen.HatchGen_PointsOnElement_begin(self)

    def end(self) -> "NCollection_Sequence< HatchGen_PointOnElement >::iterator":
        return _HatchGen.HatchGen_PointsOnElement_end(self)

    def cbegin(self) -> "NCollection_Sequence< HatchGen_PointOnElement >::const_iterator":
        return _HatchGen.HatchGen_PointsOnElement_cbegin(self)

    def cend(self) -> "NCollection_Sequence< HatchGen_PointOnElement >::const_iterator":
        return _HatchGen.HatchGen_PointsOnElement_cend(self)

    def __init__(self, *args):
        _HatchGen.HatchGen_PointsOnElement_swiginit(self, _HatchGen.new_HatchGen_PointsOnElement(*args))

    def Size(self) -> "Standard_Integer":
        return _HatchGen.HatchGen_PointsOnElement_Size(self)

    def Length(self) -> "Standard_Integer":
        return _HatchGen.HatchGen_PointsOnElement_Length(self)

    def Lower(self) -> "Standard_Integer":
        return _HatchGen.HatchGen_PointsOnElement_Lower(self)

    def Upper(self) -> "Standard_Integer":
        return _HatchGen.HatchGen_PointsOnElement_Upper(self)

    def IsEmpty(self) -> "Standard_Boolean":
        return _HatchGen.HatchGen_PointsOnElement_IsEmpty(self)

    def Reverse(self) -> "void":
        return _HatchGen.HatchGen_PointsOnElement_Reverse(self)

    def Exchange(self, I: "Standard_Integer const", J: "Standard_Integer const") -> "void":
        return _HatchGen.HatchGen_PointsOnElement_Exchange(self, I, J)

    @staticmethod
    def delNode(theNode: "NCollection_SeqNode *", theAl: "opencascade::handle< NCollection_BaseAllocator > &") -> "void":
        return _HatchGen.HatchGen_PointsOnElement_delNode(theNode, theAl)

    def Clear(self, theAllocator: "opencascade::handle< NCollection_BaseAllocator > const &"=0) -> "void":
        return _HatchGen.HatchGen_PointsOnElement_Clear(self, theAllocator)

    def Assign(self, theOther: "HatchGen_PointsOnElement") -> "NCollection_Sequence< HatchGen_PointOnElement > &":
        return _HatchGen.HatchGen_PointsOnElement_Assign(self, theOther)

    def Set(self, theOther: "HatchGen_PointsOnElement") -> "NCollection_Sequence< HatchGen_PointOnElement > &":
        return _HatchGen.HatchGen_PointsOnElement_Set(self, theOther)

    def Remove(self, *args) -> "void":
        return _HatchGen.HatchGen_PointsOnElement_Remove(self, *args)

    def Append(self, *args) -> "void":
        return _HatchGen.HatchGen_PointsOnElement_Append(self, *args)

    def Prepend(self, *args) -> "void":
        return _HatchGen.HatchGen_PointsOnElement_Prepend(self, *args)

    def InsertBefore(self, *args) -> "void":
        return _HatchGen.HatchGen_PointsOnElement_InsertBefore(self, *args)

    def InsertAfter(self, *args) -> "void":
        return _HatchGen.HatchGen_PointsOnElement_InsertAfter(self, *args)

    def Split(self, theIndex: "Standard_Integer const", theSeq: "HatchGen_PointsOnElement") -> "void":
        return _HatchGen.HatchGen_PointsOnElement_Split(self, theIndex, theSeq)

    def First(self) -> "HatchGen_PointOnElement const &":
        return _HatchGen.HatchGen_PointsOnElement_First(self)

    def ChangeFirst(self) -> "HatchGen_PointOnElement &":
        return _HatchGen.HatchGen_PointsOnElement_ChangeFirst(self)

    def Last(self) -> "HatchGen_PointOnElement const &":
        return _HatchGen.HatchGen_PointsOnElement_Last(self)

    def ChangeLast(self) -> "HatchGen_PointOnElement &":
        return _HatchGen.HatchGen_PointsOnElement_ChangeLast(self)

    def Value(self, theIndex: "Standard_Integer const") -> "HatchGen_PointOnElement const &":
        return _HatchGen.HatchGen_PointsOnElement_Value(self, theIndex)

    def ChangeValue(self, theIndex: "Standard_Integer const") -> "HatchGen_PointOnElement &":
        return _HatchGen.HatchGen_PointsOnElement_ChangeValue(self, theIndex)

    def __call__(self, *args) -> "HatchGen_PointOnElement &":
        return _HatchGen.HatchGen_PointsOnElement___call__(self, *args)

    def SetValue(self, theIndex: "Standard_Integer const", theItem: "HatchGen_PointOnElement") -> "void":
        return _HatchGen.HatchGen_PointsOnElement_SetValue(self, theIndex, theItem)
    __swig_destroy__ = _HatchGen.delete_HatchGen_PointsOnElement

    def __len__(self):
        return self.Size()


# Register HatchGen_PointsOnElement in _HatchGen:
_HatchGen.HatchGen_PointsOnElement_swigregister(HatchGen_PointsOnElement)

def HatchGen_PointsOnElement_delNode(theNode: "NCollection_SeqNode *", theAl: "opencascade::handle< NCollection_BaseAllocator > &") -> "void":
    return _HatchGen.HatchGen_PointsOnElement_delNode(theNode, theAl)

class HatchGen_PointsOnHatching(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def begin(self) -> "NCollection_Sequence< HatchGen_PointOnHatching >::iterator":
        return _HatchGen.HatchGen_PointsOnHatching_begin(self)

    def end(self) -> "NCollection_Sequence< HatchGen_PointOnHatching >::iterator":
        return _HatchGen.HatchGen_PointsOnHatching_end(self)

    def cbegin(self) -> "NCollection_Sequence< HatchGen_PointOnHatching >::const_iterator":
        return _HatchGen.HatchGen_PointsOnHatching_cbegin(self)

    def cend(self) -> "NCollection_Sequence< HatchGen_PointOnHatching >::const_iterator":
        return _HatchGen.HatchGen_PointsOnHatching_cend(self)

    def __init__(self, *args):
        _HatchGen.HatchGen_PointsOnHatching_swiginit(self, _HatchGen.new_HatchGen_PointsOnHatching(*args))

    def Size(self) -> "Standard_Integer":
        return _HatchGen.HatchGen_PointsOnHatching_Size(self)

    def Length(self) -> "Standard_Integer":
        return _HatchGen.HatchGen_PointsOnHatching_Length(self)

    def Lower(self) -> "Standard_Integer":
        return _HatchGen.HatchGen_PointsOnHatching_Lower(self)

    def Upper(self) -> "Standard_Integer":
        return _HatchGen.HatchGen_PointsOnHatching_Upper(self)

    def IsEmpty(self) -> "Standard_Boolean":
        return _HatchGen.HatchGen_PointsOnHatching_IsEmpty(self)

    def Reverse(self) -> "void":
        return _HatchGen.HatchGen_PointsOnHatching_Reverse(self)

    def Exchange(self, I: "Standard_Integer const", J: "Standard_Integer const") -> "void":
        return _HatchGen.HatchGen_PointsOnHatching_Exchange(self, I, J)

    @staticmethod
    def delNode(theNode: "NCollection_SeqNode *", theAl: "opencascade::handle< NCollection_BaseAllocator > &") -> "void":
        return _HatchGen.HatchGen_PointsOnHatching_delNode(theNode, theAl)

    def Clear(self, theAllocator: "opencascade::handle< NCollection_BaseAllocator > const &"=0) -> "void":
        return _HatchGen.HatchGen_PointsOnHatching_Clear(self, theAllocator)

    def Assign(self, theOther: "HatchGen_PointsOnHatching") -> "NCollection_Sequence< HatchGen_PointOnHatching > &":
        return _HatchGen.HatchGen_PointsOnHatching_Assign(self, theOther)

    def Set(self, theOther: "HatchGen_PointsOnHatching") -> "NCollection_Sequence< HatchGen_PointOnHatching > &":
        return _HatchGen.HatchGen_PointsOnHatching_Set(self, theOther)

    def Remove(self, *args) -> "void":
        return _HatchGen.HatchGen_PointsOnHatching_Remove(self, *args)

    def Append(self, *args) -> "void":
        return _HatchGen.HatchGen_PointsOnHatching_Append(self, *args)

    def Prepend(self, *args) -> "void":
        return _HatchGen.HatchGen_PointsOnHatching_Prepend(self, *args)

    def InsertBefore(self, *args) -> "void":
        return _HatchGen.HatchGen_PointsOnHatching_InsertBefore(self, *args)

    def InsertAfter(self, *args) -> "void":
        return _HatchGen.HatchGen_PointsOnHatching_InsertAfter(self, *args)

    def Split(self, theIndex: "Standard_Integer const", theSeq: "HatchGen_PointsOnHatching") -> "void":
        return _HatchGen.HatchGen_PointsOnHatching_Split(self, theIndex, theSeq)

    def First(self) -> "HatchGen_PointOnHatching const &":
        return _HatchGen.HatchGen_PointsOnHatching_First(self)

    def ChangeFirst(self) -> "HatchGen_PointOnHatching &":
        return _HatchGen.HatchGen_PointsOnHatching_ChangeFirst(self)

    def Last(self) -> "HatchGen_PointOnHatching const &":
        return _HatchGen.HatchGen_PointsOnHatching_Last(self)

    def ChangeLast(self) -> "HatchGen_PointOnHatching &":
        return _HatchGen.HatchGen_PointsOnHatching_ChangeLast(self)

    def Value(self, theIndex: "Standard_Integer const") -> "HatchGen_PointOnHatching const &":
        return _HatchGen.HatchGen_PointsOnHatching_Value(self, theIndex)

    def ChangeValue(self, theIndex: "Standard_Integer const") -> "HatchGen_PointOnHatching &":
        return _HatchGen.HatchGen_PointsOnHatching_ChangeValue(self, theIndex)

    def __call__(self, *args) -> "HatchGen_PointOnHatching &":
        return _HatchGen.HatchGen_PointsOnHatching___call__(self, *args)

    def SetValue(self, theIndex: "Standard_Integer const", theItem: "HatchGen_PointOnHatching") -> "void":
        return _HatchGen.HatchGen_PointsOnHatching_SetValue(self, theIndex, theItem)
    __swig_destroy__ = _HatchGen.delete_HatchGen_PointsOnHatching

    def __len__(self):
        return self.Size()


# Register HatchGen_PointsOnHatching in _HatchGen:
_HatchGen.HatchGen_PointsOnHatching_swigregister(HatchGen_PointsOnHatching)

def HatchGen_PointsOnHatching_delNode(theNode: "NCollection_SeqNode *", theAl: "opencascade::handle< NCollection_BaseAllocator > &") -> "void":
    return _HatchGen.HatchGen_PointsOnHatching_delNode(theNode, theAl)

class HatchGen_Domain(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates an infinite domain.

        Returns
        -------
        None
        Creates a domain for the curve associated to a hatching.

        Parameters
        ----------
        P1: HatchGen_PointOnHatching
        P2: HatchGen_PointOnHatching

        Returns
        -------
        None
        Creates a semi-infinite domain for the curve associated to a hatching. the `first' flag means that the given point is the first one.

        Parameters
        ----------
        P: HatchGen_PointOnHatching
        First: bool

        Returns
        -------
        None

        """
        _HatchGen.HatchGen_Domain_swiginit(self, _HatchGen.new_HatchGen_Domain(*args))

    def Dump(self, *args) -> "void":
        r"""
        Dump of the domain.

        Parameters
        ----------
        Index: int,optional
        	default value is 0

        Returns
        -------
        None

        """
        return _HatchGen.HatchGen_Domain_Dump(self, *args)

    def FirstPoint(self, *args) -> "HatchGen_PointOnHatching const &":
        r"""
        Returns the first point of the domain. the exception domainerror is raised if hasfirstpoint returns false.

        Returns
        -------
        HatchGen_PointOnHatching

        """
        return _HatchGen.HatchGen_Domain_FirstPoint(self, *args)

    def HasFirstPoint(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if the domain has a first point.

        Returns
        -------
        bool

        """
        return _HatchGen.HatchGen_Domain_HasFirstPoint(self, *args)

    def HasSecondPoint(self, *args) -> "Standard_Boolean":
        r"""
        Returns true if the domain has a second point.

        Returns
        -------
        bool

        """
        return _HatchGen.HatchGen_Domain_HasSecondPoint(self, *args)

    def SecondPoint(self, *args) -> "HatchGen_PointOnHatching const &":
        r"""
        Returns the second point of the domain. the exception domainerror is raised if hassecondpoint returns false.

        Returns
        -------
        HatchGen_PointOnHatching

        """
        return _HatchGen.HatchGen_Domain_SecondPoint(self, *args)

    def SetFirstPoint(self, *args) -> "void":
        r"""
        Sets the first point of the domain.

        Parameters
        ----------
        P: HatchGen_PointOnHatching

        Returns
        -------
        None
        Sets the first point of the domain at the infinite.

        Returns
        -------
        None

        """
        return _HatchGen.HatchGen_Domain_SetFirstPoint(self, *args)

    def SetPoints(self, *args) -> "void":
        r"""
        Sets the first and the second points of the domain.

        Parameters
        ----------
        P1: HatchGen_PointOnHatching
        P2: HatchGen_PointOnHatching

        Returns
        -------
        None
        Sets the first and the second points of the domain as the infinite.

        Returns
        -------
        None

        """
        return _HatchGen.HatchGen_Domain_SetPoints(self, *args)

    def SetSecondPoint(self, *args) -> "void":
        r"""
        Sets the second point of the domain.

        Parameters
        ----------
        P: HatchGen_PointOnHatching

        Returns
        -------
        None
        Sets the second point of the domain at the infinite.

        Returns
        -------
        None

        """
        return _HatchGen.HatchGen_Domain_SetSecondPoint(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _HatchGen.delete_HatchGen_Domain

# Register HatchGen_Domain in _HatchGen:
_HatchGen.HatchGen_Domain_swigregister(HatchGen_Domain)

class HatchGen_IntersectionPoint(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def Dump(self, *args) -> "void":
        r"""
        Dump of the point on element.

        Parameters
        ----------
        Index: int,optional
        	default value is 0

        Returns
        -------
        None

        """
        return _HatchGen.HatchGen_IntersectionPoint_Dump(self, *args)

    def Index(self, *args) -> "Standard_Integer":
        r"""
        Returns the index of the supporting curve.

        Returns
        -------
        int

        """
        return _HatchGen.HatchGen_IntersectionPoint_Index(self, *args)

    def Parameter(self, *args) -> "Standard_Real":
        r"""
        Returns the parameter on the curve.

        Returns
        -------
        float

        """
        return _HatchGen.HatchGen_IntersectionPoint_Parameter(self, *args)

    def Position(self, *args) -> "TopAbs_Orientation":
        r"""
        Returns the position of the point on the curve.

        Returns
        -------
        TopAbs_Orientation

        """
        return _HatchGen.HatchGen_IntersectionPoint_Position(self, *args)

    def SegmentBeginning(self, *args) -> "Standard_Boolean":
        r"""
        Returns the flag that the point is the beginning of a segment.

        Returns
        -------
        bool

        """
        return _HatchGen.HatchGen_IntersectionPoint_SegmentBeginning(self, *args)

    def SegmentEnd(self, *args) -> "Standard_Boolean":
        r"""
        Returns the flag that the point is the end of a segment.

        Returns
        -------
        bool

        """
        return _HatchGen.HatchGen_IntersectionPoint_SegmentEnd(self, *args)

    def SetIndex(self, *args) -> "void":
        r"""
        Sets the index of the supporting curve.

        Parameters
        ----------
        Index: int

        Returns
        -------
        None

        """
        return _HatchGen.HatchGen_IntersectionPoint_SetIndex(self, *args)

    def SetParameter(self, *args) -> "void":
        r"""
        Sets the parameter on the curve.

        Parameters
        ----------
        Parameter: float

        Returns
        -------
        None

        """
        return _HatchGen.HatchGen_IntersectionPoint_SetParameter(self, *args)

    def SetPosition(self, *args) -> "void":
        r"""
        Sets the position of the point on the curve.

        Parameters
        ----------
        Position: TopAbs_Orientation

        Returns
        -------
        None

        """
        return _HatchGen.HatchGen_IntersectionPoint_SetPosition(self, *args)

    def SetSegmentBeginning(self, *args) -> "void":
        r"""
        Sets the flag that the point is the beginning of a segment.

        Parameters
        ----------
        State: bool,optional
        	default value is Standard_True

        Returns
        -------
        None

        """
        return _HatchGen.HatchGen_IntersectionPoint_SetSegmentBeginning(self, *args)

    def SetSegmentEnd(self, *args) -> "void":
        r"""
        Sets the flag that the point is the end of a segment.

        Parameters
        ----------
        State: bool,optional
        	default value is Standard_True

        Returns
        -------
        None

        """
        return _HatchGen.HatchGen_IntersectionPoint_SetSegmentEnd(self, *args)

    def SetStateAfter(self, *args) -> "void":
        r"""
        Sets the transition state after the intersection.

        Parameters
        ----------
        State: TopAbs_State

        Returns
        -------
        None

        """
        return _HatchGen.HatchGen_IntersectionPoint_SetStateAfter(self, *args)

    def SetStateBefore(self, *args) -> "void":
        r"""
        Sets the transition state before the intersection.

        Parameters
        ----------
        State: TopAbs_State

        Returns
        -------
        None

        """
        return _HatchGen.HatchGen_IntersectionPoint_SetStateBefore(self, *args)

    def StateAfter(self, *args) -> "TopAbs_State":
        r"""
        Returns the transition state after of the intersection.

        Returns
        -------
        TopAbs_State

        """
        return _HatchGen.HatchGen_IntersectionPoint_StateAfter(self, *args)

    def StateBefore(self, *args) -> "TopAbs_State":
        r"""
        Returns the transition state before the intersection.

        Returns
        -------
        TopAbs_State

        """
        return _HatchGen.HatchGen_IntersectionPoint_StateBefore(self, *args)

    __repr__ = _dumps_object


# Register HatchGen_IntersectionPoint in _HatchGen:
_HatchGen.HatchGen_IntersectionPoint_swigregister(HatchGen_IntersectionPoint)

class HatchGen_PointOnElement(HatchGen_IntersectionPoint):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        ---purpose; creates an empty point on element.

        Returns
        -------
        None
        Creates a point from an intersection point.

        Parameters
        ----------
        Point: IntRes2d_IntersectionPoint

        Returns
        -------
        None

        """
        _HatchGen.HatchGen_PointOnElement_swiginit(self, _HatchGen.new_HatchGen_PointOnElement(*args))

    def IntersectionType(self, *args) -> "HatchGen_IntersectionType":
        r"""
        Returns the intersection type at this point.

        Returns
        -------
        HatchGen_IntersectionType

        """
        return _HatchGen.HatchGen_PointOnElement_IntersectionType(self, *args)

    def IsDifferent(self, *args) -> "Standard_Boolean":
        r"""
        Tests if the point is different from an other.

        Parameters
        ----------
        Point: HatchGen_PointOnElement
        Confusion: float

        Returns
        -------
        bool

        """
        return _HatchGen.HatchGen_PointOnElement_IsDifferent(self, *args)

    def IsIdentical(self, *args) -> "Standard_Boolean":
        r"""
        Tests if the point is identical to an other. that is to say : p1.myindex = p2.myindex abs (p1.myparam - p2.myparam) <= confusion p1.myposit = p2.myposit p1.mybefore = p2.mybefore p1.myafter = p2.myafter p1.mysegbeg = p2.mysegbeg p1.mysegend = p2.mysegend p1.mytype = p2.mytype.

        Parameters
        ----------
        Point: HatchGen_PointOnElement
        Confusion: float

        Returns
        -------
        bool

        """
        return _HatchGen.HatchGen_PointOnElement_IsIdentical(self, *args)

    def SetIntersectionType(self, *args) -> "void":
        r"""
        Sets the intersection type at this point.

        Parameters
        ----------
        Type: HatchGen_IntersectionType

        Returns
        -------
        None

        """
        return _HatchGen.HatchGen_PointOnElement_SetIntersectionType(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _HatchGen.delete_HatchGen_PointOnElement

# Register HatchGen_PointOnElement in _HatchGen:
_HatchGen.HatchGen_PointOnElement_swigregister(HatchGen_PointOnElement)

class HatchGen_PointOnHatching(HatchGen_IntersectionPoint):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates an empty point.

        Returns
        -------
        None
        Creates a point from an intersection point.

        Parameters
        ----------
        Point: IntRes2d_IntersectionPoint

        Returns
        -------
        None

        """
        _HatchGen.HatchGen_PointOnHatching_swiginit(self, _HatchGen.new_HatchGen_PointOnHatching(*args))

    def AddPoint(self, *args) -> "void":
        r"""
        Adds a point on element to the point.

        Parameters
        ----------
        Point: HatchGen_PointOnElement
        Confusion: float

        Returns
        -------
        None

        """
        return _HatchGen.HatchGen_PointOnHatching_AddPoint(self, *args)

    def ClrPoints(self, *args) -> "void":
        r"""
        Removes all the points on element of the point.

        Returns
        -------
        None

        """
        return _HatchGen.HatchGen_PointOnHatching_ClrPoints(self, *args)

    def IsEqual(self, *args) -> "Standard_Boolean":
        r"""
        Tests if the point is equal to an other. a point on hatching p1 is said to be equal to an other p2 if : | p2.myparam - p1.myparam | <= confusion.

        Parameters
        ----------
        Point: HatchGen_PointOnHatching
        Confusion: float

        Returns
        -------
        bool

        """
        return _HatchGen.HatchGen_PointOnHatching_IsEqual(self, *args)

    def IsGreater(self, *args) -> "Standard_Boolean":
        r"""
        Tests if the point is greater than an other. a point on hatching p1 is said to be greater than an other p2 if : p1.myparam - p2.myparam > confusion.

        Parameters
        ----------
        Point: HatchGen_PointOnHatching
        Confusion: float

        Returns
        -------
        bool

        """
        return _HatchGen.HatchGen_PointOnHatching_IsGreater(self, *args)

    def IsLower(self, *args) -> "Standard_Boolean":
        r"""
        Tests if the point is lower than an other. a point on hatching p1 is said to be lower than an other p2 if : p2.myparam - p1.myparam > confusion.

        Parameters
        ----------
        Point: HatchGen_PointOnHatching
        Confusion: float

        Returns
        -------
        bool

        """
        return _HatchGen.HatchGen_PointOnHatching_IsLower(self, *args)

    def NbPoints(self, *args) -> "Standard_Integer":
        r"""
        Returns the number of elements intersecting the hatching at this point.

        Returns
        -------
        int

        """
        return _HatchGen.HatchGen_PointOnHatching_NbPoints(self, *args)

    def Point(self, *args) -> "HatchGen_PointOnElement const &":
        r"""
        Returns the index-th point on element of the point. the exception outofrange is raised if index > nbpoints.

        Parameters
        ----------
        Index: int

        Returns
        -------
        HatchGen_PointOnElement

        """
        return _HatchGen.HatchGen_PointOnHatching_Point(self, *args)

    def RemPoint(self, *args) -> "void":
        r"""
        Removes the index-th point on element of the point. the exception outofrange is raised if index > nbpoints.

        Parameters
        ----------
        Index: int

        Returns
        -------
        None

        """
        return _HatchGen.HatchGen_PointOnHatching_RemPoint(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _HatchGen.delete_HatchGen_PointOnHatching

# Register HatchGen_PointOnHatching in _HatchGen:
_HatchGen.HatchGen_PointOnHatching_swigregister(HatchGen_PointOnHatching)








from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.XmlMDF import *
from OCC.Core.Message import *
from OCC.Core.TDocStd import *
from OCC.Core.XmlLDrivers import *
from OCC.Core.TCollection import *
from OCC.Core.TDF import *
from OCC.Core.XmlObjMgt import *


class xmltobjdrivers:
	@staticmethod
	def AddDrivers(aDriverTable: XmlMDF_ADriverTable, anMsgDrv: Message_Messenger) -> None: ...
	@staticmethod
	def DefineFormat(theApp: TDocStd_Application) -> None: ...
	@staticmethod
	def Factory(aGUID: Standard_GUID) -> Standard_Transient: ...

class XmlTObjDrivers_DocumentRetrievalDriver(XmlLDrivers_DocumentRetrievalDriver):
	def __init__(self) -> None: ...
	def AttributeDrivers(self, theMsgDriver: Message_Messenger) -> XmlMDF_ADriverTable: ...

class XmlTObjDrivers_DocumentStorageDriver(XmlLDrivers_DocumentStorageDriver):
	def __init__(self, theCopyright: TCollection_ExtendedString) -> None: ...
	def AttributeDrivers(self, theMsgDriver: Message_Messenger) -> XmlMDF_ADriverTable: ...

class XmlTObjDrivers_IntSparseArrayDriver(XmlMDF_ADriver):
	def __init__(self, theMessageDriver: Message_Messenger) -> None: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	@overload
	def Paste(self, theSource: XmlObjMgt_Persistent, theTarget: TDF_Attribute, theRelocTable: XmlObjMgt_RRelocationTable) -> bool: ...
	@overload
	def Paste(self, theSource: TDF_Attribute, theTarget: XmlObjMgt_Persistent, theRelocTable: XmlObjMgt_SRelocationTable) -> None: ...

class XmlTObjDrivers_ModelDriver(XmlMDF_ADriver):
	def __init__(self, theMessageDriver: Message_Messenger) -> None: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	@overload
	def Paste(self, Source: XmlObjMgt_Persistent, Target: TDF_Attribute, RelocTable: XmlObjMgt_RRelocationTable) -> bool: ...
	@overload
	def Paste(self, Source: TDF_Attribute, Target: XmlObjMgt_Persistent, RelocTable: XmlObjMgt_SRelocationTable) -> None: ...

class XmlTObjDrivers_ObjectDriver(XmlMDF_ADriver):
	def __init__(self, theMessageDriver: Message_Messenger) -> None: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	@overload
	def Paste(self, Source: XmlObjMgt_Persistent, Target: TDF_Attribute, RelocTable: XmlObjMgt_RRelocationTable) -> bool: ...
	@overload
	def Paste(self, Source: TDF_Attribute, Target: XmlObjMgt_Persistent, RelocTable: XmlObjMgt_SRelocationTable) -> None: ...

class XmlTObjDrivers_ReferenceDriver(XmlMDF_ADriver):
	def __init__(self, theMessageDriver: Message_Messenger) -> None: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	@overload
	def Paste(self, Source: XmlObjMgt_Persistent, Target: TDF_Attribute, RelocTable: XmlObjMgt_RRelocationTable) -> bool: ...
	@overload
	def Paste(self, Source: TDF_Attribute, Target: XmlObjMgt_Persistent, RelocTable: XmlObjMgt_SRelocationTable) -> None: ...

class XmlTObjDrivers_XYZDriver(XmlMDF_ADriver):
	def __init__(self, theMessageDriver: Message_Messenger) -> None: ...
	def NewEmpty(self) -> TDF_Attribute: ...
	@overload
	def Paste(self, Source: XmlObjMgt_Persistent, Target: TDF_Attribute, RelocTable: XmlObjMgt_RRelocationTable) -> bool: ...
	@overload
	def Paste(self, Source: TDF_Attribute, Target: XmlObjMgt_Persistent, RelocTable: XmlObjMgt_SRelocationTable) -> None: ...

# harray1 classes
# harray2 classes
# hsequence classes

xmltobjdrivers_AddDrivers = xmltobjdrivers.AddDrivers
xmltobjdrivers_DefineFormat = xmltobjdrivers.DefineFormat
xmltobjdrivers_Factory = xmltobjdrivers.Factory
